#ifndef _SLOT_H_
#define _SLOT_H_

#include <assert.h>
#include "Semaphore.h"
#include "SlotHandler.h"
#include "List.h"

namespace Framework
{
	template <typename Parameter> class CSlot
	{
	public:
		CSlot() :
		m_Mutex(1)
		{

		}

		~CSlot()
		{
			m_Mutex.Wait();

			while(m_Handler.Count() != 0)
			{
				delete m_Handler.Pull();
			}

			m_Mutex.Signal();
		}

		void InsertHandler(CSlotHandler<Parameter>* pHandler)
		{
			m_Mutex.Wait();

			m_Handler.Insert(pHandler, 0);
			
			m_Mutex.Signal();
		}

		void RemoveHandler(CSlotHandler<Parameter>* pHandler)
		{
			CSlotHandler<Parameter>* pStoredHandler;
			CList< CSlotHandler<Parameter> >::ITERATOR itHandler;
			bool bFound;

			m_Mutex.Wait();

			bFound = false;
			for(itHandler = m_Handler.Begin(); !itHandler; itHandler++)
			{
				pStoredHandler = (*itHandler);
				if(pStoredHandler == pHandler)
				{
					m_Handler.RemoveAt(itHandler);
					delete pStoredHandler;
					bFound = true;
					break;
				}
			}

			m_Mutex.Signal();

			if(!bFound)
			{
				assert(0);
			}

		}

		void Notify(Parameter P)
		{
			CList< CSlotHandler<Parameter> >::ITERATOR itHandler;

			m_Mutex.Wait();

			for(itHandler = m_Handler.Begin(); !itHandler; itHandler++)
			{
				(*itHandler)->Invoke(P);
			}

			m_Mutex.Signal();
		}

	private:
		CList< CSlotHandler<Parameter> >	m_Handler;
		CSemaphore							m_Mutex;
	};
}

#endif
