#include "xml/Parser.h"
#include "Types.h"

using namespace Framework;
using namespace Framework::Xml;

CParser::CParser(CStream* pStream, CNode* pRoot)
{
	m_pStream = pStream;
	m_pNode = pRoot;
	m_sText = "";
	m_nState = STATE_TEXT;
	m_nIsTagEnd = false;
	m_nTagSpace = false;
}

CParser::~CParser()
{

}

bool CParser::Parse()
{
	char nValue;
	bool nRet;
	m_pStream->Read(&nValue, 1);
	while(!m_pStream->IsEOF())
	{
		switch(m_nState)
		{
		case STATE_TEXT:
			nRet = ProcessChar_Text(nValue);
			break;
		case STATE_TAG:
			nRet = ProcessChar_Tag(nValue);
			break;
		}
		if(nRet == false)
		{
			return false;
		}
		m_pStream->Read(&nValue, 1);
	}
	return true;
}

bool CParser::ProcessChar_Text(char nChar)
{
	if(nChar == '<')
	{
		//Tag is starting

		if(strlen(m_sText) != 0)
		{
			m_pNode->InsertNode(new CNode(m_sText, false));
			m_sText = "";
		}

		m_nState = STATE_TAG;
		m_nIsTagEnd = false;
		m_nTagSpace = false;
		return true;
	}
	m_sText += nChar;
	return true;
}

bool CParser::ProcessChar_Tag(char nChar)
{
	CNode* pChild;
	if(nChar == '<')
	{
		//???
		return false;
	}
	if(nChar == '/')
	{
		//This is an end tag
		m_nIsTagEnd = true;
		return true;
	}
	if(nChar == '>')
	{
		if(m_nIsTagEnd)
		{
			//See if the tag name matches the current node's name
			if(_stricmp(m_sText, m_pNode->GetText()))
			{
				//???
				return false;
			}
			//Walk up the tree
			m_pNode = m_pNode->GetParent();
		}
		else
		{
			//Create a new node and go down
			pChild = new CNode(m_sText, true);
			m_pNode->InsertNode(pChild);
			m_pNode = pChild;
		}
		m_sText = "";
		m_nState = STATE_TEXT;
		return true;
	}
	m_sText += nChar;
	return true;
}

CNode* CParser::ParseDocument(CStream* pStream)
{
	CParser* pParser;
	CNode* pRoot;
	bool nRet;

	pRoot = new CNode();

	pParser = new CParser(pStream, pRoot);
	nRet = pParser->Parse();

	delete pParser;

	if(!nRet)
	{
		delete pRoot;
		return NULL;
	}

	return pRoot;
}
